def getActiveTCPFlows(vmanage, deviceId):
    """
    Get TCP optimized active flows from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/tcpopt/activeflows?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getExpiredTCPFlows(vmanage, deviceId):
    """
    Get TCP optimized expired flows from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/tcpopt/expiredflows?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getTCPSummary(vmanage, deviceId):
    """
    Get TCP optimization summary from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/tcpopt/summary?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getStatDataRawData(vmanage, query):
    """
    Get stats raw data
    
    Parameters:
    query	 (string):	Query string
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getStatsRawData(vmanage, statsquerystring):
    """
    Get stats raw data
    
    Parameters:
    statsquerystring:	Stats query string
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi"
    response = vmanage.apiCall("POST", endpoint, statsquerystring)
    return response
def getPostAggregationAppData(vmanage, queryfilter):
    """
    Get raw aggregated data and display applications with the highest utilization for a device
    
    Parameters:
    queryfilter:	Query filter
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/agg-app/aggregation"
    response = vmanage.apiCall("POST", endpoint, queryfilter)
    return response
def getAggregationDataByQuery(vmanage, query):
    """
    Get aggregated data based on input query and filters. The data can be filtered on time and other unique parameters based upon necessity and intended usage
    
    Parameters:
    query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/aggregation?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getPostAggregationData(vmanage, queryfilter):
    """
    Get raw aggregated data and display applications with the highest utilization for a device
    
    Parameters:
    queryfilter:	Query filter
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/aggregation"
    response = vmanage.apiCall("POST", endpoint, queryfilter)
    return response
def createDPIFlowsGridListQuery(vmanage, query, limit):
    """
    Get detailed DPI application flows list in a grid table
    
    Parameters:
    query	 (string):	Query string
	limit	 (string):	Query size
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/applications?query={query_string}&limit={limit}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def createDPISummary(vmanage, query, limit):
    """
    Get detailed DPI application flows summary
    
    Parameters:
    query	 (string):	Query string
	limit	 (string):	Query size
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/applications/summary?query={query_string}&limit={limit}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getStatDataRawDataAsCSV(vmanage, query):
    """
    Get raw data with optional query as CSV
    
    Parameters:
    query	 (string):	Query string
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/csv?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getUniqueFlowCount(vmanage, deviceId, interval, application, window, family):
    """
    Get application flow count per tunnel
    
    Parameters:
    deviceId	 (string):	Device Id
	interval	 (string):	Interval
	application	 (string):	Application
	window	 (string):	Window
	family	 (string):	Family
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/device/application/flowcount?deviceId={deviceId}&interval={interval}&application={application}&window={window}&family={family}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createDPIDeviceGridData(vmanage, query, limit):
    """
    Get detailed DPI flows list
    
    Parameters:
    query	 (string):	Query string
	limit	 (string):	Query size
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/device/applications?query={query_string}&limit={limit}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def createDPIDeviceAndAppDetails(vmanage, query):
    """
    Get detailed DPI device and app list
    
    Parameters:
    query	 (string):	Query string
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/device/details/applications?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getCount(vmanage, query):
    """
    Get response count of a query
    
    Parameters:
    query	 (string):	Query
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/doccount?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getCountPost(vmanage, query):
    """
    Get response count of a query
    
    Parameters:
    query:	Query
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/doccount"
    response = vmanage.apiCall("POST", endpoint, query)
    return response
def getStatDataFields(vmanage):
    """
    Get fields and type
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/fields"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatBulkRawData(vmanage, query, scrollId, count):
    """
    Get stats raw data
    
    Parameters:
    query	 (string):	Query string
	scrollId	 (string):	ES scroll Id
	count	 (string):	Result size
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/dpi/page?query={query_string}&scrollId={scrollId}&count={count}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getPostStatBulkRawData(vmanage, statsquerystring, scrollId, count):
    """
    Get stats raw data
    
    Parameters:
    statsquerystring:	Stats query string
	scrollId	 (string):	ES scroll Id
	count	 (string):	Result size
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/page?scrollId={scrollId}&count={count}"
    response = vmanage.apiCall("POST", endpoint, statsquerystring)
    return response
def getAggregationDataForPacketDup(vmanage, querystring):
    """
    Get time series aggregation data for packet duplication for an application over TLOCs if available
    
    Parameters:
    querystring:	Query string
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/pktdup/aggregation"
    response = vmanage.apiCall("POST", endpoint, querystring)
    return response
def getStatQueryFields(vmanage):
    """
    Get query fields
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/query/fields"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAggregationDataDPI(vmanage, querystring):
    """
    Get aggregation data and fec recovery rate if available
    
    Parameters:
    querystring:	Query string
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/dpi/recovery/aggregation"
    response = vmanage.apiCall("POST", endpoint, querystring)
    return response



def requestDNSSecActions(vmanage, action):
    """
    Request DNS-Sec actions
    
    Parameters:
    action	 (string):	DNS-Sec action
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/dnssec/actions?action={action}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def configureDNSSec(vmanage, request):
    """
    Configure DNS-Sec
    
    Parameters:
    request:	DNS sec config request
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/dnssec/config"
    response = vmanage.apiCall("POST", endpoint, request)
    return response
def getDNSSecStatus(vmanage):
    """
    Get DNS-Sec status
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/dnssec/status"
    response = vmanage.apiCall("GET", endpoint)
    return response
def configFedrampMode(vmanage, mode):
    """
    Set network deployment mode
    
    Parameters:
    mode:	Network deployment mode
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/status"
    response = vmanage.apiCall("POST", endpoint, mode)
    return response
def requestWazuhActions(vmanage, action):
    """
    Wazuh agent action
    
    Parameters:
    action	 (string):	Wazhuh Action
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/wazuh/actions?action={action}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def configureWazuhClient(vmanage, wazhuhConfig):
    """
    Configure Wazuh agent
    
    Parameters:
    wazhuhConfig:	Wazhuh configuration
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/wazuh/config"
    response = vmanage.apiCall("POST", endpoint, wazhuhConfig)
    return response
def getWazuhAgentStatus(vmanage):
    """
    Get Wazuh agent status
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/fedramp/wazuh/status"
    response = vmanage.apiCall("GET", endpoint)
    return response







def getColoGroups(vmanage):
    """
    Get COLO groups
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/cologroup"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createColoGroup(vmanage, colocationgroup):
    """
    Add COLO group
    
    Parameters:
    colocationgroup:	Colocation group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/cologroup"
    response = vmanage.apiCall("POST", endpoint, colocationgroup)
    return response

def editColoGroup(vmanage, colocationgroup, id):
    """
    Update COLO group
    
    Parameters:
    colocationgroup:	Colocation group
	id	 (string):	Colocation group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/cologroup/{id}"
    response = vmanage.apiCall("PUT", endpoint, colocationgroup)
    return response

def deleteColoGroup(vmanage, id):
    """
    Delete COLO group
    
    Parameters:
    id	 (string):	Colocation group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/cologroup/{id}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response

def getResourceGroups(vmanage):
    """
    Get all groups
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/resourcegroup"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createResourceGroup(vmanage, createagroup):
    """
    Create a group
    
    Parameters:
    createagroup:	Create a group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/resourcegroup"
    response = vmanage.apiCall("POST", endpoint, createagroup)
    return response

def switchView(vmanage, groupView):
    """
    Global netadmin switches to a different resource group view
    
    Parameters:
    groupView:	Global netadmin switches to a different resource group view
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/resourcegroup/switch"
    response = vmanage.apiCall("POST", endpoint, groupView)
    return response

def editResourceGroup(vmanage, updategroupdescription, groupId):
    """
    Update a group
    
    Parameters:
    groupdesc:  (string)	Update group description
	groupId:    (string)    Resource group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/resourcegroup/{groupId}"
    response = vmanage.apiCall("PUT", endpoint, updategroupdescription)
    return response

def deleteResourceGroup(vmanage, groupId):
    """
    Delete a group
    
    Parameters:
    groupId:    (string)    Resource group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/resourcegroup/{groupId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response

def findUsers(vmanage):
    """
    Get all users
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createUser(vmanage, user):
    """
    Create a user
    
    Parameters:
    user:	User
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user"
    response = vmanage.apiCall("POST", endpoint, user)
    return response

def getActiveSessions(vmanage):
    """
    Get active sessions
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/activeSessions"
    response = vmanage.apiCall("GET", endpoint)
    return response

def updateAdminPassword(vmanage, user):
    """
    Update admin default password
    
    Parameters:
    user:	User
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/admin/password"
    response = vmanage.apiCall("POST", endpoint, user)
    return response

def validatePassword(vmanage, userpassword):
    """
    Validate user password
    
    Parameters:
    userpassword:	User password
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/password/validate"
    response = vmanage.apiCall("POST", endpoint, userpassword)
    return response

def updatePassword(vmanage, user, userName):
    """
    Update user password
    
    Parameters:
    user:	User
	userName	 (string):	User name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/password/{userName}"
    response = vmanage.apiCall("PUT", endpoint, user)
    return response

def updateProfileLocale(vmanage, user):
    """
    Update profile locale
    
    Parameters:
    user:	User
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/profile/locale"
    response = vmanage.apiCall("PUT", endpoint, user)
    return response

def updateProfilePassword(vmanage, user):
    """
    Update profile password
    
    Parameters:
    user:	User
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/profile/password"
    response = vmanage.apiCall("PUT", endpoint, user)
    return response

def removeSessions(vmanage, user):
    """
    Remove sessions
    
    Parameters:
    user:	User
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/removeSessions"
    response = vmanage.apiCall("DELETE", endpoint, user)
    return response

def resetUser(vmanage, user):
    """
    Unlock a user
    
    Parameters:
    user:	User
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/reset"
    response = vmanage.apiCall("POST", endpoint, user)
    return response

def resourceGroupName(vmanage):
    """
    Get the name of the resource group associated with the current logged in user
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/resourceGroupName"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findUserRole(vmanage):
    """
    Check whether a user has admin role
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/role"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findUserAuthType(vmanage):
    """
    Find user authentication type, whether it is SAML enabled
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/userAuthType"
    response = vmanage.apiCall("GET", endpoint)
    return response

def updateUser(vmanage, user, userName):
    """
    Update user
    
    Parameters:
    user:	                User
	userName	 (string):	User name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/{userName}"
    response = vmanage.apiCall("PUT", endpoint, user)
    return response

def deleteUser(vmanage, userName):
    """
    Delete user
    
    Parameters:
    userName	 (string):	User name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/user/{userName}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response

def findUserGroups(vmanage):
    """
    Get all user groups
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/usergroup"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createUserGroup(vmanage, usergroup):
    """
    Create user group
    
    Parameters:
    usergroup:	User group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/usergroup"
    response = vmanage.apiCall("POST", endpoint, usergroup)
    return response

def createGroupGridColumns(vmanage):
    """
    Get user groups in a grid table
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/usergroup/definition"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findUserGroupsAsKeyValue(vmanage):
    """
    Get user groups as key value map
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/usergroup/keyvalue"
    response = vmanage.apiCall("GET", endpoint)
    return response

def updateUserGroup(vmanage, usergroup, userGroupId):
    """
    Update user group
    
    Parameters:
    usergroup:	            User group
	userGroupId	 (string):	User group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/usergroup/{userGroupId}"
    response = vmanage.apiCall("PUT", endpoint, usergroup)
    return response

def deleteUserGroup(vmanage, userGroupId):
    """
    Delete user group
    
    Parameters:
    userGroupId	 (string):	User group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/usergroup/{userGroupId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response

def getVpnGroups(vmanage):
    """
    Get VPN groups
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/vpngroup"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createVpnGroup(vmanage, vpngroup):
    """
    Add VPN group
    
    Parameters:
    vpngroup:	VPN group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/vpngroup"
    response = vmanage.apiCall("POST", endpoint, vpngroup)
    return response

def editVpnGroup(vmanage, vpngroup, id):
    """
    Update VPN group
    
    Parameters:
    vpngroup:	VPN group
	id	 (string):	VPN group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/vpngroup/{id}"
    response = vmanage.apiCall("PUT", endpoint, vpngroup)
    return response

def deleteVpnGroup(vmanage, id):
    """
    Delete VPN group
    
    Parameters:
    id	 (string):	VPN group Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/admin/vpngroup/{id}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response





def getStatDataRawAuditLogData(vmanage, inputQuery=None):
    """
    Get stat raw data
    
    Parameters:
    inputQuery	 (string):	Query filter
    
    Returns
    response    (dict)
    
    Sample input query
    
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    
    """
    query_string = vmanage.builder.generateQuery(inputQuery)
    endpoint = f"dataservice/auditlog?inputQuery={query_string}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getRawPropertyData(vmanage, query):
    """
    Get raw property data with post action
    
    Parameters:
    query:	Query filter for getting stat raw data
    
    Returns
    response    (dict)
    
    Sample query filter
    
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    """
    
    endpoint = f"dataservice/auditlog"
    response = vmanage.apiCall("POST", endpoint, query)
    return response

def getPropertyAggregationData(vmanage, inputQuery):
    """
    Get raw property data aggregated
    
    Parameters:
    inputQuery	 (string):	Query filter
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(inputQuery)
    endpoint = f"dataservice/auditlog/aggregation?inputQuery={query_string}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getPostPropertyAggregationData(vmanage, query):
    """
    Get raw property data aggregated with post action
    
    Parameters:
    query:	Query filter for getting stat raw data
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    
    """
    
    endpoint = f"dataservice/auditlog/aggregation"
    response = vmanage.apiCall("POST", endpoint, query)
    return response

def getCount(vmanage, query):
    """
    Get response count of a query
    
    Parameters:
    query	 (string):	Query
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/auditlog/doccount?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response

def getCountPost(vmanage, query):
    """
    Get response count of a query
    
    Parameters:
    query:	Query
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    """
    
    endpoint = f"dataservice/auditlog/doccount"
    response = vmanage.apiCall("POST", endpoint, query)
    return response

def getStatDataFields(vmanage):
    """
    Get fields and type
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/auditlog/fields"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getStatBulkRawPropertyData(vmanage, inputQuery, scrollId, count):
    """
    Get raw property data in bulk
    
    Parameters:
    inputQuery	 (string):	Query filter
	scrollId	 (string):	Offset of the query result
	count	 (string):	size of the query result
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    """
    query_string = vmanage.builder.generateQuery(inputQuery)
    endpoint = f"dataservice/auditlog/page?inputQuery={query_string}&scrollId={scrollId}&count={count}"
    response     = vmanage.apiCall("GET", endpoint)
    return response

def getPostStatBulkRawPropertyData(vmanage, query, scrollId, count):
    """
    Get raw property data in bulk with post action
    
    Parameters:
    query:	Query filter for getting stat raw data
	scrollId	 (string):	Offset of the query result
	count	 (string):	Size of the query result
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    
    """
    
    endpoint = f"dataservice/auditlog/page?scrollId={scrollId}&count={count}"
    response = vmanage.apiCall("POST", endpoint, query)
    return response

def getStatQueryFields(vmanage):
    """
    Get query fields
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/auditlog/query/fields"
    response = vmanage.apiCall("GET", endpoint)
    return response

def generateAuditLog(vmanage, query):
    """
    Get audit logs for last 3 hours
    
    Parameters:
    query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/auditlog/severity?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response

def getAuditSeverityCustomHistogram(vmanage, query):
    """
    Get audit log severity histogram
    
    Parameters:
    query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    Sample query filter
    q = {"size": 10000,
         "query":{
            "condition": "OR",
            "rules":[
            {
                "value":[
                    "6"
                ],
            "field":"entry_time",
            "type":"date",
            "operator":"last_n_weeks"
            }
            ]
            }
        }
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/auditlog/severity/summary?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response





def disconnect(vmanage, nmsId):
    """
    disconnect from mpd controller
    
    Parameters:
    Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/disconnect/{nmsId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def offboard(vmanage, nmsId):
    """
    offboard the mdp application
    
    Parameters:
    Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/onboard/{nmsId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def createDevice(vmanage, createdevicerequest):
    """
    Create new device
    NOTE: In a multitenant vManage system, this API is only available in the Provider view.
    
    Parameters:
    createdevicerequest:	Create device request
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device"
    response = vmanage.apiCall("POST", endpoint, createdevicerequest)
    return response

def generateBootstrapConfigForVedge(vmanage, uuid, configtype, inclDefRootCert, version):
    """
    Create vEdge device config
    
    Parameters:
    uuid	 (string):	Device uuid
	configtype	 (string):	Device config type
	inclDefRootCert	 (boolean):	Include default root certs flag
	version	 (string):	cloud-init format version
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/bootstrap/device/{uuid}?configtype={configtype}&inclDefRootCert={inclDefRootCert}&version={version}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def generateBootstrapConfigForVedges(vmanage, devicebootstraptypeandid):
    """
    Create bootstrap config for software vEdges
    
    Parameters:
    devicebootstraptypeandid:	Device bootstrap type and id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/bootstrap/devices"
    response = vmanage.apiCall("POST", endpoint, devicebootstraptypeandid)
    return response

def getBootstrapConfigZip(vmanage, id):
    """
    Download vEdge device config
    
    Parameters:
    id	 (string):	Bootstrap config id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/bootstrap/download/{id}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def generateGenericBootstrapConfigForVedges(vmanage, wanif):
    """
    Create bootstrap config for software vEdges
    
    Parameters:
    wanif	 (string):	Device WAN interface
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/bootstrap/generic/devices?wanif={wanif}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def claimDevices(vmanage, claimdevicerequest):
    """
    Claim the selected unclaimed devices
    
    Parameters:
    claimdevicerequest:	Claim device request
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/claimDevices"
    response = vmanage.apiCall("POST", endpoint, claimdevicerequest)
    return response

def getControllerVEdgeSyncStatus(vmanage):
    """
    Get controllers vEdge sync status
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/controllers/vedge/status"
    response = vmanage.apiCall("GET", endpoint)
    return response

def decommissionVedgeCloud(vmanage, uuid):
    """
    Decomission vEdge device
    
    Parameters:
    uuid	 (string):	Device uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/decommission/{uuid}"
    response = vmanage.apiCall("PUT", endpoint)
    return response

def devicesWithoutSubjectSudi(vmanage):
    """
    retrieve devices without subject sudi
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/devicesWithoutSubjectSudi"
    response = vmanage.apiCall("GET", endpoint)
    return response

def formPost(vmanage):
    """
    Upload file to vEdge
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/fileupload"
    response = vmanage.apiCall("POST", endpoint)
    return response

def generatePAYG(vmanage, bodyParameter):
    """
    Authenticate vSmart user account
    
    Parameters:
    bodyParameter:	Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/generate-payg"
    response = vmanage.apiCall("POST", endpoint, bodyParameter)
    return response

def setLifeCycle(vmanage, uuid, enable):
    """
    Set device lifecycle needed flag
    
    Parameters:
    uuid	 (string):	Device uuid
	enable	 (boolean):	lifecycle needed flag
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/lifecycle/management/{uuid}?enable={enable}"
    response = vmanage.apiCall("POST", endpoint)
    return response

def getManagementSystemIPInfo(vmanage):
    """
    Get management system IP mapping
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/management/systemip"
    response = vmanage.apiCall("GET", endpoint)
    return response

def migrateDevice(vmanage, uuid):
    """
    Migrate device software to vedge/cedge
    
    Parameters:
    uuid	 (string):	Device uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/migrateDevice/{uuid}"
    response = vmanage.apiCall("PUT", endpoint)
    return response

def resetVedgeCloud(vmanage, uuid):
    """
    Reset vEdge device
    
    Parameters:
    uuid	 (string):	Device uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/reset/{uuid}"
    response = vmanage.apiCall("PUT", endpoint)
    return response

def getRMACandidates(vmanage, deviceType, uuid):
    """
    Get RMA candidates by device type
    
    Parameters:
    deviceType	 (string):	Device Type
	uuid	 (string):	Excluded currently selected uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/rma/candidates/{deviceType}?uuid={uuid}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getRootCertStatusAll(vmanage, state):
    """
    Get controllers vEdge sync status
    
    Parameters:
    state	 (string):	Root certificate state
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/rootcertchain/status?state={state}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def checkvmanageSignedCert(vmanage):
    """
    Whether vmanage signed certificate created
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/vmanagesignedcert/iscreated"
    response = vmanage.apiCall("GET", endpoint)
    return response

def validateUser(vmanage, bodyParameter):
    """
    Authenticate vSmart user account
    
    Parameters:
    bodyParameter:	Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/smartaccount/authenticate"
    response = vmanage.apiCall("POST", endpoint, bodyParameter)
    return response

def syncDevices(vmanage, bodyParameter):
    """
    Sync devices from Smart-Account
    
    Parameters:
    bodyParameter:	Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/smartaccount/sync"
    response = vmanage.apiCall("POST", endpoint, bodyParameter)
    return response

def syncRootCertChain(vmanage):
    """
    Sync root certificate
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/sync/rootcertchain"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getTenantManagementSystemIPs(vmanage):
    """
    Get management system IP
    NOTE: In a multitenant vManage system, this API is only available in the Provider view.
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/tenant/management/systemip"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getCloudDockDataBasedOnDeviceType(vmanage, deviceCategory):
    """
    Get devices details
    
    Parameters:
    deviceCategory	 (string):	Device category
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/type/{deviceCategory}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getCloudDockDefaultConfigBasedOnDeviceType(vmanage, deviceCategory):
    """
    Get devices default config
    
    Parameters:
    deviceCategory	 (string):	Device category
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/type/{deviceCategory}/defaultConfig"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getAllUnclaimedDevices(vmanage):
    """
    Get list of all unclaimed devices
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/unclaimedDevices"
    response = vmanage.apiCall("GET", endpoint)
    return response

def invalidateVmanageRootCA(vmanage, uuid):
    """
    Invalidate vManage root CA
    
    Parameters:
    uuid	 (string):	Device uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/vmanagerootca/{uuid}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response

def getDevicesDetails(vmanage, deviceCategory, model, state, uuid, deviceIP, validity):
    """
    Get devices details. When {deviceCategory = controllers}, it returns vEdge sync status, vBond, vManage and vSmart.
 {deviceCategory = vedges}, it returns all available vEdge routers
    
    Parameters:
    deviceCategory	 (string):	Device category
	model	 (string):	Device model
	state	 (array):	List of states
	uuid	 (array):	List of device uuid
	deviceIP	 (array):	List of device system IP
	validity	 (array):	List of device validity
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/{deviceCategory}?model={model}&state={state}&uuid={uuid}&deviceIP={deviceIP}&validity={validity}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editDevice(vmanage, deviceconfig, uuid):
    """
    Edit device
    
    Parameters:
    deviceconfig:	Device config
	uuid	 (string):	Device uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/{uuid}"
    response = vmanage.apiCall("PUT", endpoint, deviceconfig)
    return response

def deleteDevice(vmanage, uuid):
    """
    Delete vEdges
    
    Parameters:
    uuid	 (string):	Device uuid
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/system/device/{uuid}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



from py_viptela.api.realtime.cellular import connection
from py_viptela.api.realtime.cellular import hardware
from py_viptela.api.realtime.cellular import network
from py_viptela.api.realtime.cellular import sim


def getConnList(vmanage, deviceId):
    """
    Get connection list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/connections?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getConnHist(vmanage, deviceId):
    """
    Get connection history list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/connectionshistory?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLocalProperties(vmanage, deviceId):
    """
    Get local properties list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/localproperties?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRevProxy(vmanage, deviceId):
    """
    Get reverse proxy mapping from vbond
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/proxymapping?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStats(vmanage, deviceId):
    """
    Get statistics from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/statistics?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getConnSummary(vmanage, deviceId):
    """
    Get connection summary from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/summary?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getValidDevicesList(vmanage, deviceId):
    """
    Get valid device list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/validvedges?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getValidVManageId(vmanage, deviceId):
    """
    Get valid vManage Id from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/validvmanageid?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getValidVSmartsList(vmanage, deviceId):
    """
    Get valid vSmart list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/orchestrator/validvsmarts?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def findSoftwareImages(vmanage):
    """
    Get software images
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createImageURL(vmanage, payload):
    """
    Create software image URL
    
    Parameters:
    payload:	Create software image request payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software"
    response = vmanage.apiCall("POST", endpoint, payload)
    return response

def getImageProps(vmanage, versionId):
    """
    Get Image Properties
    
    Parameters:
    versionId	 (string):	Version
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/imageProperties/{versionId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findImagesWithFilters(vmanage, imageType, vnfType):
    """
    Get software images
    
    Parameters:
    imageType	 (string):	Image type
	vnfType	 (string):	VNF type
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/images?imageType={imageType}&vnfType={vnfType}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getPnfProperties(vmanage, pnfType):
    """
    Get PNF Properties
    
    Parameters:
    pnfType	 (string):	PNF type
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/pnfproperties/{pnfType}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findVEdgeSoftwareVersion(vmanage):
    """
    Get vEdge software version
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/vedge/version"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findSoftwareVersion(vmanage):
    """
    Get software version
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/version"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getVnfProperties(vmanage, versionId):
    """
    Get VNF Properties
    
    Parameters:
    versionId	 (string):	Version
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/vnfproperties/{versionId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def findZtpSoftwareVersion(vmanage):
    """
    Get ZTP software version
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/ztp/version"
    response = vmanage.apiCall("GET", endpoint)
    return response

def updateImageURL(vmanage, payload, versionId):
    """
    Update software image URL
    
    Parameters:
    payload:	Update software image request payload
	versionId	 (string):	Version
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/{versionId}"
    response = vmanage.apiCall("PUT", endpoint, payload)
    return response

def deleteImageURL(vmanage, versionId):
    """
    Delete software image URL
    
    Parameters:
    versionId	 (string):	Version
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/{versionId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def getGlobal(vmanage, templateId):
    """
    Get global template
    
    Parameters:
    templateId	 (string):	Template Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/global/template/{templateId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editGlobal(vmanage, globaltemplate, templateId):
    """
    Edit global template
    
    Parameters:
    globaltemplate:	Global template
	templateId	 (string):	Template Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/global/template/{templateId}"
    response = vmanage.apiCall("PUT", endpoint, globaltemplate)
    return response

def getFeatureTempList(vmanage):
    """
    Generate device profile template list
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/profile/feature"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getProfileList(vmanage):
    """
    Generate profile template list
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/profile/template"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getDeviceProfile(vmanage, templateId):
    """
    Get device profile template
    
    Parameters:
    templateId	 (string):	Template Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/profile/template/{templateId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editDeviceProfile(vmanage, globaltemplate, templateId):
    """
    Edit device profile template
    
    Parameters:
    globaltemplate:	Global template
	templateId	 (string):	Template Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/profile/template/{templateId}"
    response = vmanage.apiCall("PUT", endpoint, globaltemplate)
    return response



import json

ERROR_CODES = {
    400: "Bad Request",
    403: "Forbidden",
    500: "Internal Server Error",
    800: "Emtpy Response",
    801: "Invalid Json Data",
    802: "Empty Data field in Response"
}

def parse_response(response):
    status_code = response.status_code
    if status_code == 200:
        if response.text != '':
            # check for invalid Json data
            try:
                raw_data = json.loads(response.text)
            except json.JSONDecodeError:
                return {"failed":True, "code": 801, "error":ERROR_CODES[801], "msg":response.text}
            else:
                if type(raw_data) == list:
                    return {"failed":False, "code": status_code, "error":"", "msg":raw_data} 
                elif raw_data.get("data"):
                    return {"failed":False, "code": status_code, "error":"", "msg":raw_data} 
                else:
                    return {"failed":True, "code": 802, "error":ERROR_CODES[802], "msg":response.text}
        else:
            # if response is an empty string, return it
            return {"failed":True, "code": 800, "error":ERROR_CODES[800], "msg":response.text}
    else:
        return {"failed":True, "code":status_code, "error":ERROR_CODES[status_code], "msg":response.text}


def getAttached(vmanage, nmsId):
    """
    Retrieve MDP attached devices
    
    Parameters:
    Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/attachDevices/{nmsId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editAttached(vmanage, devicelist, nmsId):
    """
    Edit attached devices
    
    Parameters:
    devicelist:	deviceList
	Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/attachDevices/{nmsId}"
    response = vmanage.apiCall("PUT", endpoint, devicelist)
    return response

def attach(vmanage, devicelist, nmsId):
    """
    Share devices with MDP
    
    Parameters:
    devicelist:	deviceList
	Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/attachDevices/{nmsId}"
    response = vmanage.apiCall("POST", endpoint, devicelist)
    return response

def detach(vmanage, devicelist, nmsId):
    """
    Disconnect devices from mpd controller
    
    Parameters:
    devicelist:	deviceList
	Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/detachDevices/{nmsId}"
    response = vmanage.apiCall("POST", endpoint, devicelist)
    return response

def getSupported(vmanage, nmsId):
    """
    Retrieve MDP supported devices
    
    Parameters:
    Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/mdp/devices/{nmsId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getAllDataPrefixAndFQDNLists(vmanage):
    """
    Get lists for all all data-prefix(IPv4) and Fqdn lists
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn"
    response = vmanage.apiCall("GET", endpoint)
    return response

def create(vmanage, policylist):
    """
    Create policy list
    
    Parameters:
    policylist:	Policy list
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn"
    response = vmanage.apiCall("POST", endpoint, policylist)
    return response

def preview(vmanage, policylist):
    """
    Preview a policy list based on the policy list type
    
    Parameters:
    policylist:	Policy list
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn/preview"
    response = vmanage.apiCall("POST", endpoint, policylist)
    return response

def previewById(vmanage, id):
    """
    Preview a specific policy list entry based on id provided
    
    Parameters:
    id	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn/preview/{id}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getListsById(vmanage, id):
    """
    Get a specific policy list based on the id
    
    Parameters:
    id	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn/{id}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def edit(vmanage, policylist, id):
    """
    Edit policy list entries for a specific type of policy list
    
    Parameters:
    policylist:	Policy list
	id	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn/{id}"
    response = vmanage.apiCall("PUT", endpoint, policylist)
    return response

def delete(vmanage, id):
    """
    Delete policy list entry for a specific type of policy list
    
    Parameters:
    id	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/list/dataprefixfqdn/{id}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def getAlarms(vmanage, deviceId):
    """
    Get hardware alarm list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/alarms?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getEnvironments(vmanage, deviceId):
    """
    Get hardware environment list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/environment?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getErrorAlarms(vmanage):
    """
    Get hardware error list from device
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/errors"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getInventory(vmanage, deviceId):
    """
    Get hardware inventory list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/inventory?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatusSummary(vmanage, deviceId):
    """
    Get hardware status summary from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/status/summary?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSyncAlarms(vmanage, deviceId):
    """
    Get hardware alarm list synchronously from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/synced/alarms?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSyncEnvironments(vmanage, deviceId):
    """
    Get hardware environment list synchronously from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/synced/environment?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSyncInventory(vmanage, deviceId):
    """
    Get hardware inventory list synchronously from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/synced/inventory?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSystem(vmanage, deviceId):
    """
    Get hardware system data list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/system?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getTempThreshold(vmanage, deviceId):
    """
    Get hardware temperature list from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/hardware/threshold?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getDeviceCert(vmanage, deviceId):
    """
    Get feature cert from cEdge device        
    NOTE: In a multitenant vManage system, this API is only available in the Provider and Provider-As-Tenant view.
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/featurecertificate/certificate?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def install(vmanage, certRequest):
    """
    Upload feature cert for cEdge device        
    NOTE: In a multitenant vManage system, this API is only available in the Provider and Provider-As-Tenant view.
    
    Parameters:
    certRequest:	Install feature cert request for cEdge
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/featurecertificate/certificate"
    response = vmanage.apiCall("PUT", endpoint, certRequest)
    return response

def getDeviceCsr(vmanage, deviceId):
    """
    Get CSR from cEdge device        
    NOTE: In a multitenant vManage system, this API is only available in the Provider and Provider-As-Tenant view.
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/featurecertificate/devicecsr?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def genDeviceCsr(vmanage, csrRequest):
    """
    Create CSR for cEdge device        
    NOTE: In a multitenant vManage system, this API is only available in the Provider and Provider-As-Tenant view.
    
    Parameters:
    csrRequest:	CSR request for cEdge
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/featurecertificate/devicecsr"
    response = vmanage.apiCall("PUT", endpoint, csrRequest)
    return response

def revoke(vmanage, revokeReq):
    """
    Revoke feature cert from cEdge device        
    NOTE: In a multitenant vManage system, this API is only available in the Provider and Provider-As-Tenant view.
    
    Parameters:
    revokeReq:	Revoking feature cert request for cEdge
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/featurecertificate/revoke"
    response = vmanage.apiCall("PUT", endpoint, revokeReq)
    return response

def getFeatureCaState(vmanage):
    """
    Get Feature CA state        
    NOTE: In a multitenant vManage system, this API is only available in the Provider and Provider-As-Tenant view.
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/featurecertificate/syslogconfig"
    response = vmanage.apiCall("GET", endpoint)
    return response



def installPkg(vmanage):
    """
    Install software package
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/package"
    response = vmanage.apiCall("POST", endpoint)
    return response

def getUploadImagesCount(vmanage, imageType):
    """
    Number of software image presented in vManage repository
    
    Parameters:
    imageType	 (array):	Image type
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/package/imageCount?imageType={imageType}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def downloadPkgFile(vmanage, fileName, imageType):
    """
    Download software package file
    
    Parameters:
    fileName	 (string):	Pakcage file name
	imageType	 (string):	Image type
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/package/{fileName}?imageType={imageType}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def processSoftwareImage(vmanage, imageType):
    """
    Install software image package
    
    Parameters:
    imageType	 (string):	Image type
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/package/{imageType}"
    response = vmanage.apiCall("POST", endpoint)
    return response

def getImgMeta(vmanage, versionId):
    """
    Update Package Metadata
    
    Parameters:
    versionId	 (string):	Image ID
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/package/{versionId}/metadata"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editImgMeta(vmanage, imageData, versionId):
    """
    Update Package Metadata
    
    Parameters:
    imageData:	Image Meta Data
	versionId	 (string):	Image ID
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/action/software/package/{versionId}/metadata"
    response = vmanage.apiCall("PUT", endpoint, imageData)
    return response



def getFib(vmanage, vpnId, af, prefix, tloc, color, deviceId):
    """
    Get FIB list from device (Real Time)
    
    Parameters:
    vpnId	 (string):	VPN Id
	af	 (string):	Address family
	prefix	 (string):	IP prefix
	tloc	 (string):	tloc IP
	color	 (string):	tloc color
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/fib?vpnId={vpnId}&af={af}&prefix={prefix}&tloc={tloc}&color={color}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getIetfRouting(vmanage, instanceName, af, outIf, srcProto, nextHop, deviceId):
    """
    Get ietf routing list from device
    
    Parameters:
    instanceName	 (string):	VPN Id
	af	 (string):	Address family
	outIf	 (string):	Outgoing Interface
	srcProto	 (string):	Source Protocol
	nextHop	 (string):	Next Hop Address
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/ipRoutes?instanceName={instanceName}&af={af}&outIf={outIf}&srcProto={srcProto}&nextHop={nextHop}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getIPMfibOil(vmanage, deviceId):
    """
    Get IP MFIB OIL list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/mfiboil?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getIPMfibStats(vmanage, deviceId):
    """
    Get IP MFIB statistics list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/mfibstats?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getIPMfibSummary(vmanage, deviceId):
    """
    Get IP MFIB summary list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/mfibsummary?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getNatFilter(vmanage, natVpnId, natIfname, privateSrcAddr, proto, deviceId):
    """
    Get NAT filter list from device
    
    Parameters:
    natVpnId	 (string):	NAT VPN Id
	natIfname	 (string):	NAT interface name
	privateSrcAddr	 (string):	Private source address
	proto	 (string):	Protocol
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/nat/filter?natVpnId={natVpnId}&natIfname={natIfname}&privateSrcAddr={privateSrcAddr}&proto={proto}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getNatInterface(vmanage, deviceId):
    """
    Get NAT interface list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/nat/interface?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getNatIfStats(vmanage, deviceId):
    """
    Get NAT interface statistics list from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/nat/interfacestatistics?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getNatTranslations(vmanage, deviceId):
    """
    Get NAT translation list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/nat/translation?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getNat64Translations(vmanage, deviceId):
    """
    Get NAT64 interface list from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/nat64/translation?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRouteTable(vmanage, vpnId, af, prefix, protocol, deviceId):
    """
    Get route table list from device (Real Time)
    
    Parameters:
    vpnId	 (string):	VPN Id
	af	 (string):	Address family
	prefix	 (string):	IP prefix
	protocol	 (string):	IP protocol
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/ip/routetable?vpnId={vpnId}&af={af}&prefix={prefix}&protocol={protocol}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getWorkflows(vmanage, type, id):
    """
    List all workflows for the given tenant
    
    Parameters:
    type	 (string):	Workflow type
	id	 (string):	Workflow id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/workflow?type={type}&id={id}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def saveWorkflow(vmanage, payload):
    """
    Saves the workflow
    
    Parameters:
    payload:	Request to save already created workflow with given user context
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/workflow"
    response = vmanage.apiCall("PUT", endpoint, payload)
    return response
def createWorkflow(vmanage, payload):
    """
    Creates a workflow in the system
    
    Parameters:
    payload:	Request to create workflow with given user context
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/workflow"
    response = vmanage.apiCall("POST", endpoint, payload)
    return response
def deleteWorkflow(vmanage, payload, id):
    """
    Deletes the workflow
    
    Parameters:
    payload:	Request to delete the workflow
	id	 (string):	Workflow id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/workflow?id={id}"
    response = vmanage.apiCall("DELETE", endpoint, payload)
    return response



def showInfo(vmanage):
    """
    Retrieves Certificate Signing Request information
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/setting/configuration/webserver/certificate"
    response = vmanage.apiCall("GET", endpoint)
    return response

def importCert(vmanage, signedCert):
    """
    Import a signed web server certificate
    
    Parameters:
    signedCert:	Singed certificate
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/setting/configuration/webserver/certificate"
    response = vmanage.apiCall("PUT", endpoint, signedCert)
    return response

def getCSR(vmanage, csrRequest):
    """
    Generate Certificate Signing Request
    
    Parameters:
    csrRequest:	CSR signing request
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/setting/configuration/webserver/certificate"
    response = vmanage.apiCall("POST", endpoint, csrRequest)
    return response

def dumpCert(vmanage, type):
    """
    Get certificate with alias name
    
    Parameters:
    type	 (string):	Key alias
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/setting/configuration/webserver/certificate/certificate?type={type}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getCert(vmanage):
    """
    Get certificate for alias server
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/setting/configuration/webserver/certificate/getcertificate"
    response = vmanage.apiCall("GET", endpoint)
    return response

def rollback(vmanage, type):
    """
    Rollback certificate with alias name
    
    Parameters:
    type	 (string):	Key alias
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/setting/configuration/webserver/certificate/rollback?type={type}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def get(vmanage):
    """
    Get network circuits
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/circuit"
    response = vmanage.apiCall("GET", endpoint)
    return response

def create(vmanage, networkcircuit):
    """
    Create network circuits
    
    Parameters:
    networkcircuit:	Network circuit
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/circuit"
    response = vmanage.apiCall("POST", endpoint, networkcircuit)
    return response

def edit(vmanage, networkcircuit, id):
    """
    Edit network circuits
    
    Parameters:
    networkcircuit:	Network circuit
	id	 (string):	Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/circuit/{id}"
    response = vmanage.apiCall("PUT", endpoint, networkcircuit)
    return response

def delete(vmanage, id):
    """
    Delete network circuits
    
    Parameters:
    id	 (string):	Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/networkdesign/circuit/{id}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def getConnList(vmanage, deviceId):
    """
    Get transport connection list from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/transport/connection?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getInterface(vmanage, deviceId):
    """
    Get EIGRP interface list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/eigrp/interface?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRoute(vmanage, deviceId):
    """
    Get EIGRP route from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/eigrp/route?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getTopology(vmanage, deviceId):
    """
    Get EIGRP topology info from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/eigrp/topology?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def preview(vmanage, policyassembly):
    """
    Get policy assembly preview
    
    Parameters:
    policyassembly:	Policy assembly
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/assembly/voice"
    response = vmanage.apiCall("POST", endpoint, policyassembly)
    return response


def previewById(vmanage, id):
    """
    Get policy assembly preview
    
    Parameters:
    id	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/assembly/voice/{id}"
    response = vmanage.apiCall("GET", endpoint)
    return response



from py_viptela.restclient import RestClient
from py_viptela.query_builder import Builder

class Vmanage(RestClient):
    def __init__(self, host, port, username, password):
        super().__init__(host, port, username, password)
        self.builder = Builder()


    #login
    def get_jsessionid(self,):
        payload = {"j_username":self.username,"j_password":self.password }
        res = self.session.post(f"{self.baseurl}/j_security_check", data=payload)
        if res.cookies:
            return True
        else:
            return False

    def login(self,):
        if self.get_jsessionid():
            res = self.session.get(f"{self.baseurl}/dataservice/client/token", headers=self.headers)
            self.headers['X-XSRF-TOKEN'] = res.text
            return True, "Authentication Successful."
        else:
            return False, "Authentication Failed."


from py_viptela.api import admin
from py_viptela.api import backup
from py_viptela.api import cert
from py_viptela.api import cluster
from py_viptela.api import colo
from py_viptela.api import config
from py_viptela.api import datacollection
from py_viptela.api import deployment
from py_viptela.api import device
from py_viptela.api import locale
from py_viptela.api import mdp
from py_viptela.api import monitoring
from py_viptela.api import msla
from py_viptela.api import partner
from py_viptela.api import realtime
from py_viptela.api import sdavc
from py_viptela.api import smartlicense
from py_viptela.api import system
from py_viptela.api import tenant
from py_viptela.api import tshoot
from py_viptela.api import umbrella
from py_viptela.api import utility
from py_viptela.api import workflow


def getDetail(vmanage, ifname, deviceId):
    """
    Get SFP detail
    
    Parameters:
    ifname	 (string):	IF Name
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/sfp/detail?ifname={ifname}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDiag(vmanage, ifname, deviceId):
    """
    Get SFP diagnostic
    
    Parameters:
    ifname	 (string):	IF Name
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/sfp/diagnostic?ifname={ifname}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDiagAlarm(vmanage, ifname, deviceId):
    """
    Get SFP diagnostic measurement alarm
    
    Parameters:
    ifname	 (string):	IF Name
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/sfp/diagnosticMeasurementAlarm?ifname={ifname}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDiagValue(vmanage, ifname, deviceId):
    """
    Get SFP diagnostic measurement value
    
    Parameters:
    ifname	 (string):	IF Name
	deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/sfp/diagnosticMeasurementValue?ifname={ifname}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def deleteSchduledBackup(vmanage, taskId, backupInfoId):
    """
    Delete all or a specific backup file stored in vManage
    
    Parameters:
    taskId	 (string):	task id
	backupInfoId	 (string):	Local Backup Info Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/backup/backupinfo?taskId={taskId}&backupInfoId={backupInfoId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response
def getLocalBackupInfo(vmanage, localBackupInfoId):
    """
    Get a localBackupInfo record by localBackupInfoId
    
    Parameters:
    localBackupInfoId	 (string):	localBackupInfo Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/backup/backupinfo/{localBackupInfoId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def downloadBackupFile(vmanage, path):
    """
    Download a Backup File that is already stored in vManage
    
    Parameters:
    path	 (string):	File path
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/backup/download/{path}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def exportBackup(vmanage, request):
    """
    Trigger a backup of configuration database and statstics database and store it in vManage
    
    Parameters:
    request:	backup request information
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/backup/export"
    response = vmanage.apiCall("POST", endpoint, request)
    return response
def listBackup(vmanage, size):
    """
    List all backup files of a tenant stored in vManage
    
    Parameters:
    size	 (string):	size
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/backup/list?size={size}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def importScheduledBackup(vmanage):
    """
    Submit a previously backed up file and import the data and apply it to the configuraion database
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/restore/import"
    response = vmanage.apiCall("POST", endpoint)
    return response
def remoteImportBackup(vmanage, payload):
    """
    Remote import backup from a remote URL and import the data and apply it to the configuraion database
    
    Parameters:
    payload:	Request Payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/restore/remoteimport"
    response = vmanage.apiCall("POST", endpoint, payload)
    return response
def scheduleBackup(vmanage, requestInfo):
    """
    create  backup scheduler config-db and statstics database with startDateTime and persist to config-db
    
    Parameters:
    requestInfo:	schedule request information
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/schedule/create"
    response = vmanage.apiCall("POST", endpoint, requestInfo)
    return response
def listSchedules(vmanage, limit):
    """
    Get a schedule record for backup by scheduler id
    
    Parameters:
    limit	 (integer):	size
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/schedule/list?limit={limit}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getScheduleRecordForBackup(vmanage, schedulerId):
    """
    Get a schedule record for backup by scheduler id
    
    Parameters:
    schedulerId	 (string):	scheduler id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/schedule/{schedulerId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def deleteSchedule(vmanage, schedulerId):
    """
    Delete a schedule record for backup in vManage by scheduler id
    
    Parameters:
    schedulerId	 (string):	scheduler id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/schedule/{schedulerId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def updateLeaseTime(vmanage, processId):
    """
    Update lease
    
    Parameters:
    processId	 (string):	Process Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/lock/{processId}"
    response = vmanage.apiCall("PUT", endpoint)
    return response

def removeLock(vmanage, processId):
    """
    Remove lock
    
    Parameters:
    processId	 (string):	Process Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/lock/{processId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def getHealthSummary(vmanage, type, limit, query):
    """
    Get application-aware routing statistics summary from device
    
    Parameters:
    type	 (string):	Type
	limit	 (integer):	Query result size
	query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/approute/transport/summary/{type}?limit={limit}&query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getHealth(vmanage, type, query, limit):
    """
    Get application-aware routing statistics from device
    
    Parameters:
    type	 (string):	Type
	query    (string):  Query filter
	limit	 (string):	Query result size
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/statistics/approute/transport/{type}?query={query_string}&limit={limit}"
    response     = vmanage.apiCall("GET", endpoint)
    return response



def getLogs(vmanage, deviceId):
    """
    Get device crash logs from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/crashlog?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAllLogs(vmanage):
    """
    Get device crash logs for all device
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/crashlog/details"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getInfo(vmanage, deviceId, filename):
    """
    Get device crash info from device
    
    Parameters:
    deviceId	 (string):	Device IP
	filename	 (string):	Crash file name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/crashlog/log?deviceId={deviceId}&filename={filename}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLogsSynced(vmanage, deviceId):
    """
    Get device crash logs synchronously from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/crashlog/synced?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def generatePolicyTemplateList(vmanage):
    """
    Get policy details
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createVEdgeTemplate(vmanage, templatepolicy):
    """
    Create template
    
    Parameters:
    templatepolicy:	Template policy
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge"
    response = vmanage.apiCall("POST", endpoint, templatepolicy)
    return response

def getVEdgeTemplate(vmanage, policyId):
    """
    Get template
    
    Parameters:
    policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge/definition/{policyId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getVEdgePolicyDeviceList(vmanage):
    """
    Get device list
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge/devices"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getDeviceListByPolicy(vmanage, policyId):
    """
    Get device list by policy
    
    Parameters:
    policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge/devices/{policyId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editVEdgeTemplate(vmanage, templatepolicy, policyId):
    """
    Edit template
    
    Parameters:
    templatepolicy:	Template policy
	policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge/{policyId}"
    response = vmanage.apiCall("PUT", endpoint, templatepolicy)
    return response

def deleteVEdgeTemplate(vmanage, policyId):
    """
    Delete template
    
    Parameters:
    policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge/{policyId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response

def changePolicyResourceGroup(vmanage, policyId, resourceGroupName):
    """
    Change policy resource group
    
    Parameters:
    policyId	 (string):	Policy Id
	resourceGroupName	 (string):	Resrouce group name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/vedge/{resourceGroupName}/{policyId}"
    response = vmanage.apiCall("POST", endpoint)
    return response



def getAffinityConfig(vmanage, deviceId):
    """
    Get affinity config from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/affinity/config?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAffinityStatus(vmanage, deviceId):
    """
    Get affinity status from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/affinity/status?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createRealTimeConnectionList(vmanage, peerType, sysIp, deviceId):
    """
    Get connections list from device (Real Time)
    
    Parameters:
    peerType	    (string):	Peer type
	sysIp	        (string):	Peer system IP
	deviceId	    (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/connections?peerType={peerType}&sysIp={sysIp}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createConnectionHistoryListRealTime(vmanage, peerType, sysIp, deviceId):
    """
    Get connections history list from device (Real Time)
    
    Parameters:
    peerType	 (string):	Peer type
	sysIp	 (string):	Peer system IP
	deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/connectionshistory?peerType={peerType}&sysIp={sysIp}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getTotalCountForDeviceStates(vmanage, isCached):
    """
    Get number of vedges and vsmart device in different control states
    
    Parameters:
    isCached	 (boolean):	Device State cached
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/count?isCached={isCached}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createLinkList(vmanage, state):
    """
    Get connections list
    
    Parameters:
    state	 (string):	Device State
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/links?state={state}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createLocalPropertiesListListRealTIme(vmanage, deviceId):
    """
    Get local properties list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/localproperties?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def networkSummary(vmanage, state):
    """
    Get list of unreachable devices
    
    Parameters:
    state	 (string):	Device State
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/networksummary?state={state}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getConnectionStatistics(vmanage, deviceId):
    """
    Get connection statistics from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/statistics?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLocalDeviceStatus(vmanage):
    """
    Get local device status
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/status"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createConnectionsSummary(vmanage, deviceId):
    """
    Get connections summary from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/summary?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDeviceControlStatusSummary(vmanage, deviceId):
    """
    Get device control status summary
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/summary/device?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createSyncedConnectionList(vmanage, peerType, sysIp, deviceId):
    """
    Get connections list from vManage
    
    Parameters:
    peerType	 (string):	Peer type
	sysIp	 (string):	Peer system IP
	deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/synced/connections?peerType={peerType}&sysIp={sysIp}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createLocalPropertiesSyncedList(vmanage, deviceId):
    """
    Get local properties list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/synced/localproperties?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createWanInterfaceSyncedList(vmanage, deviceId):
    """
    Get WAN interface list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/synced/waninterface?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createValidDevicesListRealTime(vmanage, deviceId):
    """
    Get valid device list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/validdevices?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getValidVManageIdRealTime(vmanage, deviceId):
    """
    Get valid vManage from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/validvmanageid?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createValidVSmartsListRealTime(vmanage, deviceId):
    """
    Get valid vSmart list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/validvsmarts?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createWanInterfaceListList(vmanage, deviceId):
    """
    Get WAN interface list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/waninterface?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPortHopColor(vmanage, deviceId):
    """
    Get port hop colors
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/control/waninterface/color?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getSlaList(vmanage, deviceId):
    """
    Get SLA class list from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/app-route/sla-class?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatList(vmanage, remote_system_ip, local_color, remote_color, deviceId):
    """
    Get application-aware routing statistics from device (Real Time)
    
    Parameters:
    remote-system-ip	 (string):	Remote system IP
	local-color	 (string):	Local color
	remote-color	 (string):	Remote color
	deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/app-route/statistics?remote-system-ip={remote_system_ip}&local-color={local_color}&remote-color={remote_color}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def fetchList(vmanage, processType):
    """
    fetchList Description
    
    Parameters:
    Parameter Description
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/download/{processType}/fetchvManageList"
    response = vmanage.apiCall("GET", endpoint)
    return response
def download(vmanage, processType, fileType, queue, deviceIp, token, fileName):
    """
    Downloading stats file
    
    Parameters:
    processType	 (string):	Process type
	fileType	 (string):	File type
	queue	 (string):	Queue name
	deviceIp	 (string):	Device IP
	token	 (string):	Token
	fileName	 (string):	File name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/download/{processType}/file/{fileType}/{queue}/{deviceIp}/{token}/{fileName}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def downloadList(vmanage, bodyParameter, processType):
    """
    Downloading list of stats file
    
    Parameters:
    bodyParameter:	Description
	processType	 (string):	Possible types are: remoteprocessing, dr
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/download/{processType}/filelist"
    response = vmanage.apiCall("POST", endpoint, bodyParameter)
    return response



def getDetailById(vmanage, clusterId):
    """
    Provide details of ids of existing clusters
    
    Parameters:
    clusterId	 (string):	Cluster Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/cluster?clusterId={clusterId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getConfigById(vmanage, clusterId):
    """
    Provide details of devices of clusters
    
    Parameters:
    clusterId	 (string):	Cluster Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/cluster/config?clusterId={clusterId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPortMappingById(vmanage, clusterId):
    """
    Provide details of port mappings in the cluster
    
    Parameters:
    clusterId	 (string):	Cluster Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/cluster/portView?clusterId={clusterId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDeviceDetailById(vmanage, deviceId):
    """
    List details for Device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/device?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSystemStatusById(vmanage, deviceId):
    """
    List all connected VNF to a device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/device/system?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getvnfById(vmanage, deviceId):
    """
    List all VNF attached with Device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/device/vnf?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def listNetworkFunctionMap(vmanage):
    """
    Retrieve network function listing
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/networkfunction/listmap"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getpnfDetails(vmanage, clusterId):
    """
    List all PNF by cluster Id
    
    Parameters:
    clusterId	 (string):	Cluster Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/pnf?clusterId={clusterId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPNFConfig(vmanage, pnfSerialNumber, clusterId):
    """
    List configuration of PNF
    
    Parameters:
    pnfSerialNumber	 (string):	PNF serial number
	clusterId	 (string):	Cluster Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/pnf/configuration?pnfSerialNumber={pnfSerialNumber}&clusterId={clusterId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getServiceChainDetails(vmanage, clusterId, userGroupName):
    """
    List all service chain or service chains by Id
    
    Parameters:
    clusterId	 (string):	Cluster Id
	userGroupName	 (string):	UserGroup Name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/servicechain?clusterId={clusterId}&userGroupName={userGroupName}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getServiceGroupById(vmanage, clusterId):
    """
    List all attached serviceGroups to cluster
    
    Parameters:
    clusterId	 (string):	Cluster Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/servicegroup?clusterId={clusterId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getvnfDetails(vmanage, clusterId, userGroupName):
    """
    Provide details of all existing VNF
    
    Parameters:
    clusterId	 (string):	Cluster Id
	userGroupName	 (string):	UserGroup Name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/vnf?clusterId={clusterId}&userGroupName={userGroupName}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def vnfActions(vmanage, vmName, deviceId, action):
    """
    VNF action
    
    Parameters:
    vmName	 (string):	VM Name
	deviceId	 (string):	Device Id
	action	 (string):	Action
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/vnf/action?vmName={vmName}&deviceId={deviceId}&action={action}"
    response = vmanage.apiCall("POST", endpoint)
    return response
def getVNFEventsCountDetail(vmanage, user_group):
    """
    Get event detail of VNF
    
    Parameters:
    user_group	 (string):	user group name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/vnf/alarms?user_group={user_group}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getVNFAlarmCount(vmanage, user_group):
    """
    Get event detail of VNF
    
    Parameters:
    user_group	 (string):	user group name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/vnf/alarms/count?user_group={user_group}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getVNFEventsDetail(vmanage, vnfName):
    """
    Get event detail of VNF
    
    Parameters:
    vnfName	 (string):	VNF name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/vnf/events?vnfName={vnfName}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getVNFInterfaceDetail(vmanage, vnfName, deviceIp, deviceClass):
    """
    Get interface detail of VNF
    
    Parameters:
    vnfName	 (string):	VNF name
	deviceIp	 (string):	Device IP
	deviceClass	 (string):	Device class
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/colocation/monitor/vnf/interface?vnfName={vnfName}&deviceIp={deviceIp}&deviceClass={deviceClass}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def generateVoiceTemplateList(vmanage):
    """
    Generate template list
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice"
    response = vmanage.apiCall("GET", endpoint)
    return response

def createVoiceTemplate(vmanage, policytemplate):
    """
    Create Template
    
    Parameters:
    policytemplate:	Policy template
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice"
    response = vmanage.apiCall("POST", endpoint, policytemplate)
    return response

def getTemplateById(vmanage, policyId):
    """
    Get templates by policy Id
    
    Parameters:
    policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/definition/{policyId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getVoicePolicyDeviceList(vmanage):
    """
    Get all device list
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/devices"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getDeviceListByPolicyId(vmanage, policyId):
    """
    Get device list by policy Id
    
    Parameters:
    policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/devices/{policyId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def generateVoicePolicySummary(vmanage):
    """
    Get templates that map a device model
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/summary"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getVoiceTemplatesForDevice(vmanage, deviceModel):
    """
    Get templates that map a device model
    
    Parameters:
    deviceModel:  Device Model
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/{deviceModel}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editVoiceTemplate(vmanage, policytemplate, policyId):
    """
    Edit Template
    
    Parameters:
    policytemplate:	Policy template
	policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/{policyId}"
    response = vmanage.apiCall("PUT", endpoint, policytemplate)
    return response

def deleteVoiceTemplate(vmanage, policyId):
    """
    Delete Template
    
    Parameters:
    policyId	 (string):	Policy Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/voice/{policyId}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def MapTrafficProfiles(vmanage, apppayload):
    """
    Set SLA class for policy cloud discovered applications
    
    Parameters:
    apppayload:	App payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/clouddiscoveredapp"
    response = vmanage.apiCall("POST", endpoint, apppayload)
    return response

def getCustomApps(vmanage):
    """
    Get all policy custom applications
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/customapp"
    response = vmanage.apiCall("GET", endpoint)
    return response

def create(vmanage, apppayload):
    """
    Create a policy custom applications
    
    Parameters:
    apppayload:	App payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/customapp"
    response = vmanage.apiCall("POST", endpoint, apppayload)
    return response

def getById(vmanage, id):
    """
    Get a policy custom applications
    
    Parameters:
    id	 (string):	Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/customapp/{id}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def edit(vmanage, apppayload, id):
    """
    Edit a policy custom applications
    
    Parameters:
    apppayload:	App payload
	id	 (string):	Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/customapp/{id}"
    response = vmanage.apiCall("PUT", endpoint, apppayload)
    return response

def delete(vmanage, id):
    """
    Delete a policy custom applications
    
    Parameters:
    id	 (string):	Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/policy/customapp/{id}"
    response = vmanage.apiCall("DELETE", endpoint)
    return response



def getStatDataRawData(vmanage, query):
    """
    Get stats raw data
    
    Parameters:
    query	 (string):	Query string
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getStatsRawData(vmanage, statsquerystring):
    """
    Get stats raw data
    
    Parameters:
    statsquerystring:	Stats query string
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event"
    response = vmanage.apiCall("POST", endpoint, statsquerystring)
    return response
def getAggregationDataByQuery(vmanage, query):
    """
    Get aggregated data based on input query and filters. The data can be filtered on time and other unique parameters based upon necessity and intended usage
    
    Parameters:
    query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event/aggregation?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getPostAggregationDataByQuery(vmanage, statsquerystring):
    """
    Get aggregated data based on input query and filters. The data can be filtered on time and other unique parameters based upon necessity and intended usage
    
    Parameters:
    statsquerystring:	Stats query string
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/aggregation"
    response = vmanage.apiCall("POST", endpoint, statsquerystring)
    return response
def getPostAggregationAppDataByQuery(vmanage, statsquerystring):
    """
    Get aggregated data based on input query and filters. The data can be filtered on time and other unique parameters based upon necessity and intended usage
    
    Parameters:
    statsquerystring:	Stats query string
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/app-agg/aggregation"
    response = vmanage.apiCall("POST", endpoint, statsquerystring)
    return response
def getComponentsAsKeyValue(vmanage):
    """
    Retrieve components as key/value pairs
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/component/keyvalue"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatDataRawDataAsCSV(vmanage, query):
    """
    Get raw data with optional query as CSV
    
    Parameters:
    query	 (string):	Query string
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event/csv?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getCount(vmanage, query):
    """
    Get response count of a query
    
    Parameters:
    query	 (string):	Query
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event/doccount?query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getCountPost(vmanage, query):
    """
    Get response count of a query
    
    Parameters:
    query:	Query
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/doccount"
    response = vmanage.apiCall("POST", endpoint, query)
    return response
def enableEventsFromFile(vmanage):
    """
    Set enable events from file flag
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/enable/fileprocess"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatDataFields(vmanage):
    """
    Get fields and type
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/fields"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getListenersInfo(vmanage):
    """
    Retrieve listener information
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/listeners"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatBulkRawData(vmanage, query, scrollId, count):
    """
    Get stats raw data
    
    Parameters:
    query	 (string):	Query string
	scrollId	 (string):	ES scroll Id
	count	 (string):	Result size
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event/page?query={query_string}&scrollId={scrollId}&count={count}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getPostStatBulkRawData(vmanage, statsquerystring, scrollId, count):
    """
    Get stats raw data
    
    Parameters:
    statsquerystring:	Stats query string
	scrollId	 (string):	ES scroll Id
	count	 (string):	Result size
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/page?scrollId={scrollId}&count={count}"
    response = vmanage.apiCall("POST", endpoint, statsquerystring)
    return response
def getStatQueryFields(vmanage):
    """
    Get query fields
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/query/fields"
    response = vmanage.apiCall("GET", endpoint)
    return response
def createEventsQueryConfig(vmanage):
    """
    Create query configuration
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/query/input"
    response = vmanage.apiCall("GET", endpoint)
    return response
def findEvents(vmanage, level, deviceId, query):
    """
    Retrieve events
    
    Parameters:
    level	 (array):	Severity level
	deviceId	 (string):	Device IP
	query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event/severity?severity-level={level}&deviceId={deviceId}&query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getSeverityHistogram(vmanage, deviceId, query):
    """
    Retrieve severity histogram
    
    Parameters:
    deviceId	 (string):	Device IP
	query	 (string):	Query filter
    
    Returns
    response    (dict)
    
    
    """
    query_string = vmanage.builder.generateQuery(query)
    endpoint = f"dataservice/event/severity/summary?deviceId={deviceId}&query={query_string}"
    response     = vmanage.apiCall("GET", endpoint)
    return response
def getEventTypesAsKeyValue(vmanage):
    """
    Retrieve event types as key/value pairs
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/event/types/keyvalue"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getCloudXRecv(vmanage, deviceId):
    """
    Get CloudExpress routes received list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/cloudx?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLinkList(vmanage, state):
    """
    Get OMP connection list
    
    Parameters:
    state	 (string):	Connection state
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/links?state={state}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMcastAdvt(vmanage, deviceId):
    """
    Get OMP multicast autodiscover advertised list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/mcastautodiscoveradvt?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMcastRecv(vmanage, deviceId):
    """
    Get OMP multicast autodiscover received list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/mcastautodiscoverrecv?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMcastRoutesAdvt(vmanage, deviceId):
    """
    Get OMP multicast routes advertised list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/mcastroutesadvt?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMcastRoutesRecv(vmanage, deviceId):
    """
    Get OMP multicast routes received list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/mcastroutesrecv?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSessionList(vmanage, deviceId):
    """
    Get OMP session list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/peers?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAdvRoutesList(vmanage, deviceId):
    """
    Get OMP advertised routes list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/routes/advertised?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAdvRoutesListV6(vmanage, deviceId):
    """
    Get OMP advertised routes list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/routes/advertised/ompIpv6?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRcvdRouteList(vmanage, deviceId):
    """
    Get OMP received routes list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/routes/received?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRcvdRouteListVv6(vmanage, deviceId):
    """
    Get OMP received routes list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/routes/received/ompIpv6?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getServices(vmanage, deviceId):
    """
    Get OMP services list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/services?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDeviceStatus(vmanage):
    """
    Get device OMP status
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/status"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSummary(vmanage, deviceId):
    """
    Get OMP summary
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/summary?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSyncSessionList(vmanage, deviceId):
    """
    Get OP session list
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/synced/peers?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAdvTlocsList(vmanage, deviceId):
    """
    Get advertised TLOCs list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/tlocs/advertised?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRcvdTlocsList(vmanage, deviceId):
    """
    Get received TLOCs list (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/omp/tlocs/received?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getDspActive(vmanage, deviceId):
    """
    Get DSPFarm Active DSP info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/dspActive?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPhoneInfo(vmanage, deviceId):
    """
    Get phone registration info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/phoneInfo?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDSPFarmProfiles(vmanage, deviceId):
    """
    Get DSPFarm Profiles info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/profiles?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSccpCcmGroups(vmanage, deviceId):
    """
    Get DSPFarm SCCP CCM Groups info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/sccpCcmGroups?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSccpConnections(vmanage, deviceId):
    """
    Get DSPFarm SCCP Connections info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/sccpConnections?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getVoiceCalls(vmanage, deviceId):
    """
    Get voice call info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/voiceCalls?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getVoipCalls(vmanage, deviceId):
    """
    Get VOIP call info from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voice/voipCalls?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getIsdnStatus(vmanage, deviceId):
    """
    Retrieve Voice ISDN Status from device
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voiceisdninfo/isdnstatus?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getControllerInfo(vmanage, deviceId):
    """
    Retrieve T1E1 controller last 15 min stats from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voicet1e1controllerinfo/current15minstats?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getControllerStats(vmanage, deviceId):
    """
    Retrieve T1E1 controller total stats from device (Real Time)
    
    Parameters:
    deviceId	 (string):	Device Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/voicet1e1controllerinfo/totalstats?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getCortexStatus(vmanage):
    """
    Get Cortex List
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex"
    response = vmanage.apiCall("GET", endpoint)
    return response

def authAzureCredAndAdd(vmanage, credential):
    """
    Authenticate Cloud Account Credentials
    
    Parameters:
    credential:	Credential
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/cloud/authenticate"
    response = vmanage.apiCall("POST", endpoint, credential)
    return response

def getMappedWanRG(vmanage, accountid, cloudregion):
    """
    Get Mapped WAN Resource Groups
    
    Parameters:
    accountid	 (string):	Account Id
	cloudregion	 (string):	Cloud region
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/map?accountid={accountid}&cloudregion={cloudregion}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def syncWanRG(vmanage, wanRG):
    """
    Sync WAN Resource Groups
    
    Parameters:
    wanRG:	WAN resource group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/sync"
    response = vmanage.apiCall("POST", endpoint, wanRG)
    return response

def getWanRG(vmanage, accountid):
    """
    Get WAN Resource Groups
    
    Parameters:
    accountid	 (string):	Account Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/wanrg?accountid={accountid}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def editWanRG(vmanage, wanRG):
    """
    Edit WAN Resource Groups
    
    Parameters:
    wanRG:	WAN resource group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/wanrg"
    response = vmanage.apiCall("PUT", endpoint, wanRG)
    return response

def saveWanRG(vmanage, wanRG):
    """
    Create WAN Resource Groups
    
    Parameters:
    wanRG:	WAN resource group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/wanrg"
    response = vmanage.apiCall("POST", endpoint, wanRG)
    return response

def deleteWanRG(vmanage, wanRG):
    """
    Delete WAN Resource Groups
    
    Parameters:
    wanRG:	WAN resource group
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/template/cortex/wanrg"
    response = vmanage.apiCall("DELETE", endpoint, wanRG)
    return response



def getVmanageControlStatus(vmanage, isCached, vpnId):
    """
    Retrieve vManage control status
    
    Parameters:
    isCached	 (boolean):	Is cached flag
	vpnId	 (array):	VPN Id
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/network/connectionssummary?isCached={isCached}&vpnId={vpnId}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getRebootCount(vmanage, isCached):
    """
    Retrieve reboot count
    
    Parameters:
    isCached	 (boolean):	Is cached flag
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/network/issues/rebootcount?isCached={isCached}"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getNetworkIssuesSummary(vmanage):
    """
    Retrieve network issues summary
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/network/issues/summary"
    response = vmanage.apiCall("GET", endpoint)
    return response

def getNetworkStatusSummary(vmanage):
    """
    Retrieve network status summary
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/network/status"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getEvalInfo(vmanage, deviceId):
    """
    Get license evaluation info from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/license/evaluation?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPAKInfo(vmanage, deviceId):
    """
    Get license pak info from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/license/pak?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPrivacyInfo(vmanage, deviceId):
    """
    Get license privacy info from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/license/privacy?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRegInfo(vmanage, deviceId):
    """
    Get license registration info from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/license/registration?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getUDIInfo(vmanage, deviceId):
    """
    Get license UDI info from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/license/udi?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getUsageInfo(vmanage, deviceId):
    """
    Get license usage info from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/license/usage?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



import requests
from py_viptela.response_parser import parse_response


class RestClient:
    """
    TODO: methods for this class will be implemented last
    """

    def __init__(self, host, port, username, password):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.session = requests.Session()
        self.session.verify = False
        self.headers = {'Content-Type':'application/json'}
        self.baseurl = f"https://{self.host}:{self.port}"

    def apiCall(self, method, endpoint, payload=None):
        methods = {
            "GET":self.session.get,
            "POST": self.session.post,
            "PUT": self.session.put,
            "DELETE": self.session.delete
        }
        url = f"{self.baseurl}/{endpoint}"

        try:
            response = methods[method](url=url, headers=self.headers, data=payload)
        except requests.exceptions.SSLError:
            return{'error': 'SSL Error'}
        except requests.exceptions.ConnectTimeout:
            return{'error': 'Connection Timeout'}
        except requests.exceptions.ConnectionError:
            return {'error': 'Connection Failed'}
        except requests.exceptions.Timeout:
            return {'error': 'Timeout'}
        else:
            return parse_response(response)


def editDeviceWithLicense(vmanage, license):
    """
    Update device subscription
    
    Parameters:
    license:	Subscription and license
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/assignLicense"
    response = vmanage.apiCall("PUT", endpoint, license)
    return response
def createDeviceWithLicense(vmanage, license):
    """
    Create device subscription
    
    Parameters:
    license:	Subscription and license
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/assignLicense"
    response = vmanage.apiCall("POST", endpoint, license)
    return response
def getMSLADevices(vmanage):
    """
    Retrieve devices subscription
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/devices"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getDeviceDistributionOverview(vmanage):
    """
    Get device distribution 
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/devices/assignmentdistribution"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLicenseandDeviceOverview(vmanage):
    """
    Get license device overview 
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/devices/licenseanddeviceoverviewtable"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLicenseDistributionOverview(vmanage):
    """
    Get license distribution 
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/devices/licensedistribution"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSubscriptionOverview(vmanage):
    """
    Get subscription overview 
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/devices/subscriptiondistribution"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLicenseSubscriptions(vmanage, payload):
    """
    Retrieve MSLA subscription/licenses
    
    Parameters:
    payload:	Request Payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/licenses/subscription"
    response = vmanage.apiCall("POST", endpoint, payload)
    return response
def syncLicenses(vmanage, synclicense):
    """
    Retrieve MSLA subscription/licenses
    
    Parameters:
    synclicense:	Sync license
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/licenses/sync"
    response = vmanage.apiCall("POST", endpoint, synclicense)
    return response
def getlicensedDeviceCount(vmanage):
    """
    get license and device count
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/monitoring/licensedDeviceCount"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getLicensedDistributionDetails(vmanage):
    """
    get license and device count
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/monitoring/licensedDistributionDetails"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getpackagingDistributionDetails(vmanage):
    """
    get packaging distribution details
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/monitoring/packagingDistributionDetails"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getSubscriptions(vmanage):
    """
    Retrieve subscriptions
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/subscriptions"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getAllTemplate(vmanage):
    """
    Retrieve all MSLA template
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/template"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getTempLicenseSubscriptions(vmanage, payload):
    """
    Retrieve MSLA subscription/licenses
    
    Parameters:
    payload:	Request Payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/template/licenses"
    response = vmanage.apiCall("POST", endpoint, payload)
    return response
def getVaLicenseSubscriptions(vmanage, payload):
    """
    Retrieve MSLA subscription/licenses
    
    Parameters:
    payload:	Request Payload
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/msla/va/License"
    response = vmanage.apiCall("POST", endpoint, payload)
    return response



def getStatisticType(vmanage):
    """
    Get statistics types
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics"
    response = vmanage.apiCall("GET", endpoint)
    return response
def startStatsCollection(vmanage):
    """
    Start stats collect
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/collect"
    response = vmanage.apiCall("GET", endpoint)
    return response
def generateStatsCollectThreadReport(vmanage):
    """
    Get stats collect thread report
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/collect/thread/status"
    response = vmanage.apiCall("GET", endpoint)
    return response
def resetStatsCollection(vmanage, processQueue):
    """
    Reset stats collect thread report
    
    Parameters:
    processQueue	 (integer):	Process queue
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/collection/reset/{processQueue}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def enableStatsDemo(vmanage, enable):
    """
    Enable statistic demo mode
    
    Parameters:
    enable	 (boolean):	Demo mode flag
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/demomode?enable={enable}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def processStatsData(vmanage):
    """
    Process stats data
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/process"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatsProcessingCounters(vmanage):
    """
    Get statistics processing counters
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/process/counters"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatsProcessReport(vmanage, processQueue):
    """
    Get stats process report
    
    Parameters:
    processQueue	 (integer):	Process queue
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/process/status?processQueue={processQueue}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getStatsProcessThreadReport(vmanage):
    """
    Get stats process thread report
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/process/thread/status"
    response = vmanage.apiCall("GET", endpoint)
    return response



from py_viptela.api.config import assembler
from py_viptela.api.config import azure
from py_viptela.api.config import builder
from py_viptela.api.config import cexp
from py_viptela.api.config import circuits
from py_viptela.api.config import cor
from py_viptela.api.config import dashboard
from py_viptela.api.config import device
from py_viptela.api.config import disaster
from py_viptela.api.config import featurecert
from py_viptela.api.config import multicloud
from py_viptela.api.config import networkdesign
from py_viptela.api.config import segments
from py_viptela.api.config import settings
from py_viptela.api.config import sig
from py_viptela.api.config import softwareactions
from py_viptela.api.config import template
from py_viptela.api.config import topology


def getPnicStats(vmanage, deviceId):
    """
    Get pnic interfaces from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/csp/pnic?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getPNICStatsFromDevice(vmanage, deviceId):
    """
    Get pnic stats from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/csp/pnic/synced?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getRBACInterface(vmanage, deviceId):
    """
    Get RBAC interfaces from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/csp/rbac?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getMpDatabase(vmanage, deviceId):
    """
    Get mp database from device
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/cfm/mp/database?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMpLocalMep(vmanage, domain, service, mepId, deviceId):
    """
    Get mp local mep from device
    
    Parameters:
    domain	 (string):	Domain Name
	service	 (string):	Service Name
	mepId	 (number):	MEP ID
	deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/cfm/mp/local/mep?domain={domain}&service={service}&mepId={mepId}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMpLocalMip(vmanage, level, port, svcInst, deviceId):
    """
    Get mp local mip from device
    
    Parameters:
    level	 (number):	Level
	port	 (string):	Port
	svcInst	 (number):	Service Instance
	deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/cfm/mp/local/mip?level={level}&port={port}&svcInst={svcInst}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def getMpRemoteMep(vmanage, domain, service, locMepId, remMepId, deviceId):
    """
    Get mp remote mep from device
    
    Parameters:
    domain	 (string):	Domain Name
	service	 (string):	Service Name
	locMepId	 (number):	Local MEP ID
	remMepId	 (number):	Remote MEP ID
	deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/device/cfm/mp/remotemep?domain={domain}&service={service}&locMepId={locMepId}&remMepId={remMepId}&deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response



def getDisabledDeviceList(vmanage, indexName):
    """
    Get list of disabled devices for a statistics index
    
    Parameters:
    indexName	 (string):	Index name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/settings/disable/devicelist/{indexName}"
    response = vmanage.apiCall("GET", endpoint)
    return response
def updateDeviceList(vmanage, disableddevice, indexName):
    """
    Update list of disabled devices for a statistics index
    
    Parameters:
    disableddevice:	Disabled device
	indexName	 (string):	Index name
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/settings/disable/devicelist/{indexName}"
    response = vmanage.apiCall("PUT", endpoint, disableddevice)
    return response
def getSettings(vmanage):
    """
    Get statistics settings
    
    Parameters:
            
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/settings/status"
    response = vmanage.apiCall("GET", endpoint)
    return response
def updateSettings(vmanage, statssetting):
    """
    Update statistics settings
    
    Parameters:
    statssetting:	Stats setting
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/settings/status"
    response = vmanage.apiCall("PUT", endpoint, statssetting)
    return response
def getEnabledIndex(vmanage, deviceId):
    """
    Get list of enabled device for statistics index
    
    Parameters:
    deviceId	 (string):	Device IP
    
    Returns
    response    (dict)
    
    
    """
    
    endpoint = f"dataservice/statistics/settings/status/device?deviceId={deviceId}"
    response = vmanage.apiCall("GET", endpoint)
    return response
